'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.revertChange = revertChange;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Take a change event and revert it
function revertChange(change) {
  var obs = change.object;
  if (change.type === 'splice') {
    var removed = obs.splice.apply(obs, [change.index, change.addedCount].concat((0, _toConsumableArray3.default)(change.removed)));
    return {
      removed: removed,
      type: 'splice',
      index: change.index,
      object: change.object,
      addedCount: change.removed.length
    };
  }

  if (change.type === 'update') {
    var newChange = {
      object: change.object,
      type: 'update',
      name: change.name,
      index: change.index,
      oldValue: null
    };

    if (change.index !== undefined) {
      // Array
      newChange.oldValue = obs[change.index];
      obs[change.index] = change.oldValue;
    } else if (obs.constructor.name === 'ObservableMap') {
      // MobX Map
      newChange.oldValue = obs.get(change.name);
      obs.set(change.name, change.oldValue);
    } else {
      // MobX Object
      newChange.oldValue = obs[change.name];
      obs[change.name] = change.oldValue;
    }

    return newChange;
  }

  if (change.type === 'add') {
    if (obs.constructor.name === 'ObservableMap') {
      change.object.delete(change.name);
    } else {
      delete change.object[change.name];
    }

    return {
      type: 'delete',
      name: change.name,
      object: change.object,
      oldValue: change.newValue
    };
  }

  if (obs.constructor.name === 'ObservableMap') {
    change.object.set(change.name, change.oldValue);
  } else {
    change.object[change.name] = change.oldValue;
  }

  return {
    type: 'add',
    name: change.name,
    object: change.object,
    newValue: change.oldValue
  };
}